// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

#include "xmacpp/FilterSession.h"
#include "SessionBase.h"
#include "FilterThread.h"

namespace xma {
//////////////////////////////////////////////////////////////////////////////
class InternalFilterSession : virtual public FilterSession, virtual public FilterThread<> {
public:
  InternalFilterSession(IntSessionFactoryPtr sf, XmaFilterProperties* props);
  virtual ~InternalFilterSession();

  virtual int32_t Enqueue(queue_t& data) override;
  virtual int32_t Dequeue(queue_t& data) override;

private:
  virtual int32_t Process(queue_t& in, queue_t& out, EmptyDynamicParams* dynamic_params) override;

protected:
  using vpi_mem_handle_t = std::unique_ptr<void, std::function<void(VpiMemHandle mem_ctx)>>;
  using filter_handle_t  = std::unique_ptr<void, std::function<void(void*)>>;
  vpi_mem_handle_t m_mem_ctx;
  filter_handle_t  m_handle;
  XmaFilterType    m_type;
};

using IntFilterSessionPtr = std::shared_ptr<InternalFilterSession>;

}; // Namespace xma
