// Copyright (C) 2022 - 2023, Xilinx Inc - All rights reserved
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may
// not use this file except in compliance with the License. A copy of the
// License is located at
//
//      http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
// WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
// License for the specific language governing permissions and limitations
// under the License.

#pragma once

#include <dlfcn.h>
#include <errno.h>
#include <memory.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <termios.h>
#include <unistd.h>

#include <xma.h>

#include "xlnx_ml_arg_parse.h"
#include "xlnx_ml_common.h"
#include "xlnx_ml_xma_props.h"

typedef struct XlnxMLCtx {
    XlnxMLProps            ml_params;
    XmaMioFilterProperties ml_xma_props;
    XmaFilterProperties    xma_upload_props;
    XmaFrameProperties     upload_fprops;
    XmaFilterProperties    xma_download_props;
    XmaFrameProperties     download_fprops;
    XmaLogHandle           log;
    XmaHandle              handle;
    XmaMioFilterSession*   ml_session;
    XmaFilterSession*      upload_session;
    XmaFilterSession*      download_sessions;
    XmaFrame*              input_xframe[1];
    XmaFrameProperties     input_fprops;
    XmaFrame*              output_xframe_list[1];
    XmaFrameProperties     output_fprops;
    size_t                 num_frames_inferenced;
    int                    pts;
} XlnxMLCtx;

/**
 * xlnx_ml_cleanup_ml_ctx: Cleanup the ml context - free
 * xma frames, destroy ml session, free xrm resources.
 * @param ctx: The ml context
 */
void xlnx_ml_cleanup_ml_ctx(XlnxMLCtx* ctx);

/**
 * xlnx_ml_device_init: Get/allocate xrm resources, xma
 * initialize.
 *
 * @param ctx: The ml context
 * @return XMA_APP_SUCCESS on success XMA_APP_ERROR on error
 */
int32_t xlnx_ml_device_init(XlnxMLCtx* ctx);

/**
 * xlnx_ml_create_ml_ctx: Uses ctx->ml_params to create the
 * rest of the ml context. Does not init the device/create
 * session.
 * @param ctx: A pointer to the ml context
 * @return XMA_APP_SUCCESS on success XMA_APP_ERROR on error
 */
int32_t xlnx_ml_create_ml_ctx(XlnxMLCtx* ctx);

/**
 * xlnx_ml_create_ml_sessions: Creates the ml sessions
 * using the ml props.
 * @param ctx: A pointer to the ml context
 * @return XMA_APP_SUCCESS on success XMA_APP_ERROR on error
 */
int32_t xlnx_ml_create_ml_sessions(XlnxMLCtx* ctx);
